/**
 * Address
 * Represents a physical address
 * @public
 */
declare type Address = {
    /** Identifies the nature of the address. */
    addressType: AddressTypeCode;
    /** Identification of a division of a large organisation or building. */
    department?: string;
    /** Identification of a sub-division of a large organisation or building. */
    subDepartment?: string;
    /** Name of a street or thoroughfare. */
    streetName?: string;
    /** Number that identifies the position of a building on a street. */
    buildingNumber?: string;
    /** Name of the building or house. */
    buildingName?: string;
    /** Floor or storey within a building. */
    floor?: string;
    /** Numbered box in a post office. */
    postBox?: string;
    /** Building room number. */
    room?: string;
    /** Identifier consisting of a group of letters and/or numbers that is added to a postal address to assist the sorting of mail. */
    postcode?: string;
    /** Name of a built-up area, with defined boundaries, and a local government. */
    townName: string;
    /** Specific location name within the town. */
    townLocationName?: string;
    /** Identifies a subdivision within a country subdivision. */
    districtName?: string;
    /** Identifies a subdivision of a country such as state, region, province. */
    countrySubDivision?: string;
    /** Information that locates and identifies a specific address, presented in free format text. */
    addressLine?: string[];
    /** Nation with its own government. */
    country: ISOCountryCode;
};

/**
 * Address Type Code
 * Specifies the type of address
 * @public
 */
declare type AddressTypeCode = 'HOME' | 'BIZZ' | 'GEOG';

/**
 * Who is the agent acting on behalf of the counterparty
 * @public
 */
export declare interface Agent {
    did: DID;
    type: AgentType;
    logo?: URI;
    url?: URI;
    name?: string;
    verified?: boolean;
}

/**
 * The type of Agent. Either a wallet or a VASP
 * @public
 */
export declare enum AgentType {
    PRIVATE = "WALLET",
    VASP = "VASP"
}

/**
 * Beneficiary
 * Represents the receiver of the requested VA transfer
 * @public
 */
declare type Beneficiary = {
    /** Array of persons associated with the beneficiary */
    beneficiaryPersons?: Person[];
    /** Array of account numbers, maximum 100 characters each */
    accountNumber?: string[];
};

/**
 * Fields specific to the beneficiary of a transaction
 * @public
 */
declare type BeneficiaryFields = {
    destination?: destination;
};

/**
 * Beneficiary VASP
 * Represents the VASP which receives the VA transfer
 * @public
 */
declare type BeneficiaryVASP = {
    /** The beneficiary VASP information */
    beneficiaryVASP?: Person;
};

/**
 * A blockchain address
 * @public
 */
declare type BlockchainAddress = string;

/**
 * Chain Agnostic Account Identifier
 * Represents an account on a specific blockchain
 * Format: `{caip2}:{address}`
 * @example "eip155:1:0x742d35Cc6634C0532925a3b844Bc454e4438f44e" for an Ethereum account
 * @public
 */
declare type CAIP10 = `${CAIP2}:${string}`;

/**
 * Chain Agnostic Asset Identifier
 * Represents an asset on a specific blockchain
 * Format: `{caip2}/{asset_namespace}:{asset_reference}`
 * @example "eip155:1/erc20:0x6b175474e89094c44da98b954eedeac495271d0f" for DAI on Ethereum
 * @public
 */
declare type CAIP19 = `${CAIP2}/${string}:${string}`;

/**
 * Chain Agnostic Blockchain Identifier
 * Represents a blockchain in a chain-agnostic way
 * Format: `namespace:reference`
 * @example "eip155:1" for Ethereum Mainnet
 * @public
 */
declare type CAIP2 = `${string}:${string}`;

/**
 * Options for callback and redirect URIs
 * @public
 */
export declare interface CallbackOptions {
    callback?: URI;
    redirectUri?: URI;
}

/**
 * Represents a cancel component message
 * @public
 */
declare type Cancel = {
    type: CMType.CANCEL;
};

/**
 * Component Message Types
 * @public
 */
export declare const enum CMType {
    COMPLETE = "complete",
    RESIZE = "resize",
    RESULT = "result",
    READY = "ready",
    INVALID = "invalid",
    MODAL = "openModal",
    ERROR = "error",
    CLOSE = "closeModal",
    CANCEL = "cancel"
}

/**
 * Represents a completed component message
 * @public
 */
declare type Completed = {
    type: CMType.COMPLETE;
    response: ComponentResponse;
};

/**
 * Component Message
 * @public
 */
export declare type ComponentMessage = Completed | Cancel | Error_2 | Ready | ResizeRequest | InvalidValue;

/**
 * An abstract request for communicating with a component
 * @public
 */
declare interface ComponentRequest {
    requestId?: RequestID;
    customer?: Counterparty;
}

declare interface ComponentResponse {
    requestID: RequestID;
    valid: boolean;
    status: Status;
    errors: ValidationError[];
}

/**
 * An object representing a connection request
 * @public
 */
export declare interface ConnectionRequest extends ComponentRequest {
    asset: TransactionAsset;
}

/**
 * An object representing a connection response
 * @public
 */
export declare interface ConnectionResponse extends ComponentResponse {
    value: ConnectionRequest;
    ivms: IVMS101;
    proof?: OwnershipProof;
    txCreate?: V1Transaction;
}

/**
 * The counterparty of a transaction.
 * @public
 */
export declare interface Counterparty {
    name?: string;
    accountNumber?: string;
    did?: DID;
    type?: PersonType;
    verified?: boolean;
    geographicAddress?: Address;
    nationalIdentification?: NationalIdentification;
    website?: URI;
    phone?: string;
}

/**
 * A crypto credential
 * @public
 */
declare type CryptoCredential = `${string}.${string}.mastercard`;

/**
 * Date and Place of Birth
 * Represents the date and place of birth for a natural person
 * @public
 */
declare type DateAndPlaceOfBirth = {
    /** Date of birth in ISO 8601 format (YYYY-MM-DD) */
    dateOfBirth?: ISODate;
    /** Place of birth (max 70 characters) */
    placeOfBirth?: string;
};

/**
 * Ownership Proof using Self Declaration
 * @public
 */
export declare interface DeclarationProof extends OwnershipProof {
    type: ProofTypes.SelfDeclaration;
    attestation: string;
    confirmed: boolean;
}

/**
 * An object representing a deposit transaction
 * @public
 */
export declare interface Deposit extends OriginatorFields, Transaction {
}

/**
 * An object representing a request for a deposit
 * @public
 */
export declare interface DepositRequest extends DepositRequestFields, Transaction {
}

/**
 * Fields specific to a deposit request
 * @public
 */
declare type DepositRequestFields = {
    destinations: destination[];
};

/**
 * The destination of a transaction either a blockchain address, a CAIP-19 address, or a travel address.
 * @public
 */
export declare type destination = BlockchainAddress | CAIP10 | CryptoCredential | TravelAddress;

/**
 * A Decentralized Identifier
 * @public
 */
declare type DID = `did:${string}:${string}`;

/**
 * Digital Token Identifier
 * @public
 */
declare type DTI = string;

/**
 * An embedded Notabene component
 * @public
 */
export declare class EmbeddedComponent<V, R> {
    private _url;
    private _value;
    private _errors;
    private iframe?;
    private eventManager;
    private modal?;
    /**
     * Creates an instance of EmbeddedComponent.
     * @param url - The URL of the embedded component
     * @param value - The initial transaction value
     */
    constructor(url: string, value: V);
    /**
     * Gets the URL of the embedded component
     * @returns The URL of the embedded component
     */
    get url(): string;
    /**
     * Gets the current transaction value
     * @returns The current transaction value
     */
    get value(): V;
    get errors(): ValidationError[];
    /**
     * Opens the component URL in the current window
     */
    open(): void;
    /**
     * Mounts the component to a parent element
     * @param parentId - The ID of the parent element
     * @throws Will throw an error if the parent element is not found
     */
    mount(parentId: string): void;
    /**
     * Embeds the component into a parent element
     * @param parent - The parent element to embed the component into
     */
    embed(parent: Element): void;
    /**
     * Sends a message to the embedded component
     * @param message - The message to send
     */
    send(message: HostMessage): void;
    /**
     * Adds an event listener for a specific message type
     * @param messageType - The type of message to listen for
     * @param callback - The callback function to execute when the message is received
     */
    on(messageType: string, callback: MessageCallback): void;
    /**
     * Removes an event listener for a specific message type
     * @param messageType - The type of message to stop listening for
     * @param callback - The callback function to remove
     */
    off(messageType: string, callback: MessageCallback): void;
    /**
     * Updates the transaction value and sends an update message to the component
     * @param value - The new transaction value
     */
    update(value: V): void;
    /**
     * Waits for the component to complete and returns the transaction response
     * @returns A promise that resolves with the transaction response
     */
    completion(): Promise<R>;
    /**
     * Opens the component in a modal dialog
     */
    openModal(): Promise<R>;
    /**
     * Closes the modal dialog
     *
     */
    closeModal(): void;
}

/**
 * Represents an error component message
 * @public
 */
declare type Error_2 = {
    type: CMType.ERROR;
    message: string;
};

/**
 * Field properties
 * @public
 */
declare type FieldOptions = boolean | {
    optional: boolean;
    transmit: boolean;
};

declare type FieldTypes = {
    naturalPerson?: NaturalPersonFields;
    legalPerson?: LegalPersonFields;
};

/**
 * Host Message Types
 * @public
 */
declare const enum HMType {
    UPDATE = "update",
    REQUEST_RESPONSE = "requestResponse"
}

/**
 * Host Messages
 * @public
 */
export declare type HostMessage = UpdateValue;

/**
 * Intermediary VASP
 * Represents an intermediary Virtual Asset Service Provider
 * @public
 */
declare type IntermediaryVASP = {
    /** The intermediary VASP information */
    intermediaryVASP?: Person;
    /** The sequence number of this VASP in the transfer path */
    sequence?: number;
};

/**
 * Represents an invalid value component message
 * @public
 */
declare type InvalidValue = {
    type: CMType.INVALID;
    value: any;
    errors: ValidationError[];
};

/**
 * ISO-3166 Alpha-2 country code
 * @example "US" for United States, "GB" for United Kingdom
 * @public
 */
declare type ISOCountryCode = string;

/**
 * 3 letter ISO currency code
 * @public
 */
declare type ISOCurrency = string;

/**
 * A point in time, represented as a day within the calendar year. Compliant with ISO 8601.
 * Format: YYYY-MM-DD
 * @example "2023-05-15" for May 15, 2023
 * @public
 */
declare type ISODate = `${number}-${number}-${number}`;

/**
 * IVMS101 definition
 *
 * @public
 */
export declare type IVMS101 = {
    originator?: Originator;
    beneficiary?: Beneficiary;
    originatingVASP?: OriginatingVASP;
    beneficiaryVASP?: BeneficiaryVASP;
    transferPath?: TransferPath;
    payloadMetadata?: PayloadMetadata;
};

/**
 * Legal Person
 * Represents a legal person with all associated information
 * @public
 */
declare type LegalPerson = {
    /** The name of the legal person */
    name: LegalPersonName;
    /** The address of the legal person */
    geographicAddress?: Address[];
    /** A distinct identifier that uniquely identifies the person to the institution in context */
    customerNumber?: string;
    /** A distinct identifier used by governments to uniquely identify a legal person */
    nationalIdentification?: NationalIdentification;
    /** The country in which the legal person is registered */
    countryOfRegistration?: ISOCountryCode;
};

/**
 * Field names for LegalPerson
 * @public
 */
declare type LegalPersonFieldName = 'name' | 'lei' | 'website' | 'email' | 'phone' | 'geographicAddress' | 'nationalIdentification' | 'countryOfRegistration';

/**
 * Field properties by field name
 * @public
 */
declare type LegalPersonFields = {
    [name in LegalPersonFieldName]?: FieldOptions;
};

/**
 * Legal Person Name
 * Represents the full name structure for a legal person
 * @public
 */
declare type LegalPersonName = {
    /** Array of name identifiers */
    nameIdentifier: LegalPersonNameID[];
    /** Array of local name identifiers */
    localNameIdentifier?: LocalLegalPersonNameID[];
    /** Array of phonetic name identifiers */
    phoneticNameIdentifier?: LocalLegalPersonNameID[];
};

/**
 * Legal Person Name ID
 * Represents a name identifier for a legal person
 * @public
 */
declare type LegalPersonNameID = {
    /** Name by which the legal person is known */
    legalPersonName: string;
    /** The nature of the name specified */
    legalPersonNameIdentifierType: LegalPersonNameTypeCode;
};

/**
 * Legal Person Name Type Code
 * Specifies the type of name for a legal person
 * @public
 */
declare type LegalPersonNameTypeCode = 'LEGL' | 'SHRT' | 'TRAD';

/**
 * A LEI Legal Entity Identifier
 * @public
 */
declare type LEI = string;

/**
 * Local Legal Person Name ID
 * Represents a local name identifier for a legal person
 * @public
 */
declare type LocalLegalPersonNameID = {
    /** Name of the legal person, maximum 100 characters in local format */
    legalPersonName?: string;
    /** Type of legal person name identifier */
    legalPersonNameIdentifierType?: LegalPersonNameTypeCode;
};

/**
 * Local Natural Person Name ID
 * Represents a local name identifier for a natural person
 * @public
 */
declare type LocalNaturalPersonNameID = {
    /** Primary identifier, maximum 100 characters in local format */
    primaryIdentifier?: string;
    /** Secondary identifier, maximum 100 characters in local format */
    secondaryIdentifier?: string;
    /** Type of name identifier */
    nameIdentifierType?: NaturalPersonNameTypeCode;
};

declare type MessageCallback = (message: ComponentMessage) => void;

/**
 * Ownership Proof using Micro Transfer
 * @public
 */
export declare interface MicroTransferProof extends OwnershipProof {
    type: ProofTypes.MicroTransfer;
    txhash: string;
    chain: CAIP2;
    amountSubunits: string;
}

/**
 * National Identification
 * Represents a national identifier for a person or entity
 * @public
 */
declare type NationalIdentification = {
    /** National identifier (max 35 characters) */
    nationalIdentifier?: string;
    /** Type of national identifier */
    nationalIdentifierType?: NationalIdentifierTypeCode;
    /** Country that issued the national identifier */
    countryOfIssue?: ISOCountryCode;
    /** Registration authority (format: RA followed by 6 digits) */
    registrationAuthority?: string;
};

/**
 * National Identifier Type Code
 * Specifies the type of national identifier
 * @public
 */
declare type NationalIdentifierTypeCode = 'ARNU' | 'CCPT' | 'RAID' | 'DRLC' | 'FIIN' | 'TXID' | 'SOCS' | 'IDCD' | 'LEIX' | 'MISC';

/**
 * Natural Person
 * Represents a natural person with all associated information
 * @public
 */
declare type NaturalPerson = {
    /** The distinct words used as identification for an individual */
    name: NaturalPersonName;
    /** The particulars of a location at which a person may be communicated with */
    geographicAddress?: Address[];
    /** A distinct identifier used by governments to uniquely identify a natural person */
    nationalIdentification?: NationalIdentification;
    /** A distinct identifier that uniquely identifies the person to the institution in context */
    customerIdentification?: string;
    /** Date and place of birth of a person */
    dateAndPlaceOfBirth?: DateAndPlaceOfBirth;
    /** Country in which a person resides (the place of a person's home) */
    countryOfResidence?: ISOCountryCode;
};

/**
 * Field names for NaturalPerson
 * @public
 */
declare type NaturalPersonFieldName = 'name' | 'website' | 'email' | 'phone' | 'geographicAddress' | 'nationalIdentification' | 'dateOfBirth' | 'placeOfBirth' | 'countryOfResidence';

/**
 * Field properties by field name
 * @public
 */
declare type NaturalPersonFields = {
    [name in NaturalPersonFieldName]?: FieldOptions;
};

/**
 * Natural Person Name
 * Represents the full name structure for a natural person
 * @public
 */
declare type NaturalPersonName = {
    /** Array of name identifiers */
    nameIdentifier?: NaturalPersonNameID[];
    /** Array of local name identifiers */
    localNameIdentifier?: LocalNaturalPersonNameID[];
    /** Array of phonetic name identifiers */
    phoneticNameIdentifier?: LocalNaturalPersonNameID[];
};

/**
 * Natural Person Name ID
 * Represents a name identifier for a natural person
 * @public
 */
declare type NaturalPersonNameID = {
    /** Primary identifier, maximum 100 characters */
    primaryIdentifier?: string;
    /** Secondary identifier, maximum 100 characters */
    secondaryIdentifier?: string;
    /** Type of name identifier */
    nameIdentifierType?: NaturalPersonNameTypeCode;
};

/**
 * Natural Person Name Type Code
 * Specifies the type of name for a natural person
 *
 * @public
 */
declare type NaturalPersonNameTypeCode = 'ALIA' | 'BIRT' | 'MAID' | 'LEGL' | 'MISC';

/**
 * Primary constructor for Notabene UX elements
 *
 * This class provides methods to create and manage various Notabene components
 * such as withdrawal assist, deposit assist, connect, and deposit request.
 * It also handles URL generation and fragment decoding for these components.
 *
 * @public
 */
declare class Notabene {
    private nodeUrl?;
    private authToken?;
    private uxUrl;
    private theme?;
    private locale?;
    /**
     * Creates a new instance of the Notabene SDK
     *
     * @param config - Configuration options for the Notabene SDK
     */
    constructor(config: NotabeneConfig);
    /**
     * Generates a URL for a Notabene component
     *
     * @param path - The path of the component
     * @param value - Transaction data
     * @param configuration - Optional transaction configuration
     * @param callbacks - Optional callback configuration
     * @returns component URL
     * @internal
     */
    componentUrl<V>(path: string, value: V, configuration?: TransactionOptions, callbacks?: CallbackOptions): string;
    /**
     * Creates a new embedded component
     *
     * @param path - The path of the component
     * @param value - Transaction data
     * @param options - Optional transaction options
     * @param callbacks - Optional callback configuration
     * @returns A new EmbeddedComponent instance
     * @internal
     */
    createComponent<V, R>(path: string, value: V, options?: TransactionOptions, callbacks?: CallbackOptions): EmbeddedComponent<V, R>;
    /**
     * Creates a withdrawal assist component
     *
     * @param value - Withdrawal transaction data
     * @param options - Optional transaction options
     * @param callbacks - Optional callback configuration
     * @returns A new EmbeddedComponent instance for withdrawal assistance
     */
    createWithdrawalAssist(value: Partial<Withdrawal>, options?: TransactionOptions, callbacks?: CallbackOptions): EmbeddedComponent<Partial<Withdrawal>, TransactionResponse<Withdrawal>>;
    /**
     * Creates a deposit assist component
     *
     * @param value - Deposit transaction data
     * @param options - Optional transaction options
     * @param callbacks - Optional callback configuration
     * @returns A new EmbeddedComponent instance for deposit assistance
     * @alpha
     */
    createDepositAssist(value: Partial<Deposit>, options?: any, callbacks?: CallbackOptions): EmbeddedComponent<Partial<Deposit>, TransactionResponse<Deposit>>;
    /**
     * Creates a connect component
     *
     * @param value - Connection request data
     * @param options - Optional transaction options
     * @param callbacks - Optional callback configuration
     * @returns A new EmbeddedComponent instance for connection
     */
    createConnectWallet(value: ConnectionRequest, options?: TransactionOptions, callbacks?: CallbackOptions): EmbeddedComponent<ConnectionRequest, ConnectionResponse>;
    /**
     * Creates a deposit request component
     *
     * @param value - Deposit request data
     * @param options - Optional transaction options
     * @param callbacks - Optional callback configuration
     * @returns A new EmbeddedComponent instance for deposit requests
     * @alpha
     */
    createDepositRequest(value: DepositRequest, options?: TransactionOptions, callbacks?: CallbackOptions): EmbeddedComponent<DepositRequest, TransactionResponse<DepositRequest>>;
    /**
     * Decodes a URL fragment into an object
     *
     * @param fragment - The URL fragment to decode
     * @returns An object containing the decoded key-value pairs
     */
    decodeFragmentToObject(fragment: string): Record<string, string>;
}
export { Notabene }
export default Notabene;

/**
 * Notabene Asset Identifier
 * @public
 */
declare type NotabeneAsset = string;

/**
 * Configuration for the Notabene SDK
 *
 * @public
 */
/**
 * Configuration for the Notabene SDK
 *
 * @public
 */
export declare interface NotabeneConfig {
    /**
     * The URL of the Notabene API node
     */
    nodeUrl?: string;
    /**
     * The authentication token for the Notabene API
     */
    authToken?: string;
    /**
     * The URL of the Notabene UX components
     */
    uxUrl?: string;
    /**
     * Custom theme configuration for the UX components
     */
    theme?: Theme;
    /**
     * The locale to use for the UX components
     */
    locale?: string;
}

/**
 * Originating VASP
 * Represents the VASP which initiates the VA transfer
 * @public
 */
declare type OriginatingVASP = {
    /** The originating VASP information */
    originatingVASP?: Person;
};

/**
 * Originator
 * Represents the account holder who initiates the VA transfer
 * @public
 */
declare type Originator = {
    /** Array of persons associated with the originator */
    originatorPersons?: Person[];
    /** Array of account numbers, maximum 100 characters each */
    accountNumber?: string[];
};

/**
 * Fields specific to the originator of a transaction
 * @public
 */
declare type OriginatorFields = {
    source?: Source;
};

/**
 * Ownership Proof
 * @public
 */
export declare interface OwnershipProof {
    type: ProofTypes;
    status: ProofStatus;
    did: DID;
    address: CAIP10;
}

/**
 * 6.7
 * Data describing the contents of the payload.
 */
declare interface PayloadMetadata {
    /** The method used to map from a national system of writing to Latin script. */
    transliterationMethod?: TransliterationMethodCode[];
    /** The version of IVMS 101 to which the payload complies. */
    payloadVersion: PayloadVersionCode;
}

/**
 * The version of IVMS 101 to which the payload complies.
 */
declare enum PayloadVersionCode {
    /** Published May 2020 */
    V101 = "101",
    /** Published August 2023 */
    V101_2023 = "101.2023"
}

/**
 * Person
 * Represents either a natural person or a legal person
 * @public
 */
declare type Person = {
    /** Natural person information */
    naturalPerson?: NaturalPerson;
    /** Legal person information */
    legalPerson?: LegalPerson;
};

/**
 * The type of counterparty. Either a natural person or a legal person. If the customer is the same as the counterparty then the counterparty is a self.
 * @public
 */
export declare enum PersonType {
    NATURAL = "natural",
    LEGAL = "legal",
    SELF = "self"
}

/**
 * Status of the proof
 * @public
 */
export declare enum ProofStatus {
    PENDING = "pending",// Verification is pending
    FAILED = "rejected",// Rejected
    FLAGGED = "flagged",// Flagged for manual review
    VERIFIED = "verified"
}

/**
 * The type of Proofs supported
 * @public
 */
export declare enum ProofTypes {
    SelfDeclaration = "self-declaration",
    PersonalSignEIP191 = "eip-191",
    PersonalSignEIP712 = "eip-712",
    PersonalSignBIP137 = "bip-137",
    PersonalSignXPUB = "xpub",
    MicroTransfer = "microtransfer",
    Screenshot = "screenshot"
}

/**
 * Represents a ready component message
 * @public
 */
declare type Ready = {
    type: CMType.READY;
};

declare type RequestID = UUID;

/**
 * Represents a resize request component message
 * @public
 */
declare type ResizeRequest = {
    type: CMType.RESIZE;
    height: number;
};

/**
 * Ownership Proof using Screenshot
 * @public
 */
export declare interface ScreenshotProof extends OwnershipProof {
    type: ProofTypes.Screenshot;
    url: string;
}

/**
 * Ownership Proof using Message Signature
 * @public
 */
export declare interface SignatureProof extends OwnershipProof {
    type: ProofTypes.PersonalSignEIP191 | ProofTypes.PersonalSignEIP712 | ProofTypes.PersonalSignBIP137 | ProofTypes.PersonalSignXPUB;
    proof: string;
    attestation: string;
    wallet_provider: string;
}

/**
 * The source of a transaction
 * @public
 */
declare type Source = BlockchainAddress | CAIP10;

/**
 * The verification status of a transaction
 * @public
 */
export declare enum Status {
    EMPTY = "empty",
    VERIFY = "verify",
    PENDING = "pending",
    VERIFIED = "verified",
    BANNED = "banned"
}

/**
 * The theme of the Notabene SDK
 * @public
 */
export declare type Theme = {
    primaryColor: string;
    secondaryColor: string;
    fontFamily: string;
    logo: string;
};

/**
 * An abstract transaction object
 * @public
 */
export declare interface Transaction extends ComponentRequest {
    agent: Agent;
    counterparty: Counterparty;
    asset: TransactionAsset;
    amountDecimal: number;
    proof?: OwnershipProof;
    assetPrice?: {
        price: number;
        currency: ISOCurrency;
    };
}

/**
 * The asset of a transaction either a Notabene asset, a CAIP-19 asset, or a DTI.
 * @public
 */
export declare type TransactionAsset = NotabeneAsset | CAIP19 | DTI;

/**
 * Configuration options for Transaction components
 * @public
 */
export declare interface TransactionOptions {
    proofs?: {
        microTransfer?: {
            destination: BlockchainAddress;
            timeout?: number;
        };
        fallbacks?: ProofTypes[];
    };
    allowedAgentTypes?: AgentType[];
    allowedCounterpartyTypes?: PersonType[];
    fields?: FieldTypes;
}

/**
 * The response of a transaction
 * @public
 */
export declare interface TransactionResponse<V> extends ComponentResponse {
    value: V;
    ivms: IVMS101;
    proof?: OwnershipProof;
    txCreate?: V1Transaction;
}

/**
 * Transfer Path
 * Represents the path of intermediary VASPs in a transfer
 * @public
 */
declare type TransferPath = {
    /** Array of intermediary VASPs involved in the transfer */
    transferPath?: IntermediaryVASP[];
};

/**
 * Transliteration Method Code
 * Specifies the method used to transliterate text
 * @public
 */
declare type TransliterationMethodCode = 'arab' | 'aran' | 'armn' | 'cyrl' | 'deva' | 'geor' | 'grek' | 'hani' | 'hebr' | 'kana' | 'kore' | 'thai' | 'othr';

/**
 * A travel address
 * @public
 */
declare type TravelAddress = `ta${string}`;

/**
 * Represents an update value host message
 * @public
 */
declare type UpdateValue = {
    type: HMType.UPDATE;
    value: ComponentRequest;
};

/**
 * A Uniform Resource Identifier
 * @public
 */
declare type URI = string;

/**
 * UUID v4
 * @public
 */
declare type UUID = string;

declare type V1Asset = NotabeneAsset | {
    caip19: CAIP19;
};

/**
 * Transaction payload suitable for calling Notabene v1 tx/create
 * @public
 */
declare type V1Transaction = {
    transactionAsset: V1Asset;
    transactionAmount: string;
    originatorEqualsBeneficiary?: boolean;
    originatorVASPdid: DID;
    beneficiaryVASPdid: DID;
    beneficiaryProof: OwnershipProof;
    originator: Originator;
    beneficiary: Beneficiary;
};

/**
 * Validation error
 * @public
 */
declare type ValidationError = {
    attribute: string;
    message: string;
};

/**
 * A VASP agent acting on behalf of the counterparty
 * @public
 */
export declare interface VASP extends Agent {
    lei?: LEI;
    logo?: URI;
    website?: URI;
    countryOfRegistration?: ISOCountryCode;
}

/**
 * A wallet agent acting on behalf of the counterparty
 * @public
 */
export declare interface Wallet extends Agent {
    proof: OwnershipProof;
    wallet_connect_id?: string;
}

/**
 * An object representing a withdrawal transaction
 * @public
 */
export declare interface Withdrawal extends BeneficiaryFields, Transaction {
}

export { }
