import { fc, test } from '@fast-check/vitest';
import { describe } from 'vitest';
import { abitraryCallbackOptions, arbitraryComponent } from '../arbitraries';
import Notabene from '../notabene';
import { arbitraryLocale } from '../utils/arbitraries';

describe('Notabene', () => {
  describe('defaults', () => {
    test.prop([
      fc.string({ minLength: 3 }),
      fc.webUrl(),
      arbitraryComponent(),
      fc.object(),
    ])(
      'Should set the correct values',
      (authToken, nodeUrl, component, value) => {
        const notabene = new Notabene({
          nodeUrl,
          authToken,
        });
        const url = new URL(notabene.componentUrl(component, value));
        return (
          url.hash ===
            `#authToken=${encodeURIComponent(authToken)}&value=${encodeURIComponent(JSON.stringify(value))}` &&
          url.searchParams.get('nodeUrl') === nodeUrl &&
          url.pathname === `/${component}`
        );
      },
    );
  });

  describe('locale', () => {
    test.prop([
      fc.string({ minLength: 3 }),
      fc.webUrl(),
      arbitraryComponent(),
      fc.object(),
      arbitraryLocale(),
    ])(
      'Should set the correct values',
      (authToken, nodeUrl, component, value, locale) => {
        const notabene = new Notabene({
          nodeUrl,
          authToken,
          locale,
        });
        const url = new URL(notabene.componentUrl(component, value));
        return (
          url.hash ===
            `#authToken=${encodeURIComponent(authToken)}&value=${encodeURIComponent(JSON.stringify(value))}` &&
          url.searchParams.get('nodeUrl') === nodeUrl &&
          url.searchParams.get('locale') === locale &&
          url.pathname === `/${component}`
        );
      },
    );
  });

  describe('minimal', () => {
    test.prop([arbitraryComponent(), fc.object()])(
      'Should set the correct values',
      (component, value) => {
        const notabene = new Notabene({});
        const url = new URL(notabene.componentUrl(component, value));
        return (
          url.hash === `#value=${encodeURIComponent(JSON.stringify(value))}` &&
          url.hostname === 'connect.notabene.id' &&
          url.pathname === `/${component}`
        );
      },
    );
  });

  describe('with uxUrl', () => {
    test.prop([
      fc.string({ minLength: 3 }),
      fc.webUrl(),
      arbitraryComponent(),
      fc.webAuthority(),
      fc.object(),
    ])(
      'Should set the correct values',
      (authToken, nodeUrl, component, uxAuthority, value) => {
        const notabene = new Notabene({
          nodeUrl,
          authToken,
          uxUrl: `https://${uxAuthority}`,
        });
        const url = new URL(notabene.componentUrl(component, value));
        const [domain, port] = uxAuthority.split(':');
        return (
          url.hostname === domain &&
          url.port === (port || '') &&
          url.hash ===
            `#authToken=${encodeURIComponent(authToken)}&value=${encodeURIComponent(JSON.stringify(value))}` &&
          url.searchParams.get('nodeUrl') === nodeUrl &&
          url.pathname === `/${component}`
        );
      },
    );
  });

  describe('with value and configuration', () => {
    test.prop([
      fc.string({ minLength: 3 }),
      fc.webUrl(),
      arbitraryComponent(),
      fc.object(),
      fc.object(),
    ])(
      'Should set the correct values',
      (authToken, nodeUrl, component, value, configuration) => {
        const notabene = new Notabene({
          nodeUrl,
          authToken,
        });
        const url = new URL(
          notabene.componentUrl(component, value, configuration),
        );
        return (
          url.hash ===
            `#authToken=${encodeURIComponent(authToken)}&value=${encodeURIComponent(JSON.stringify(value))}&configuration=${encodeURIComponent(JSON.stringify(configuration))}` &&
          url.searchParams.get('nodeUrl') === nodeUrl &&
          url.pathname === `/${component}`
        );
      },
    );
  });

  describe('with value and configuration', () => {
    test.prop([
      fc.string({ minLength: 3 }),
      fc.webUrl(),
      arbitraryComponent(),
      fc.object(),
      fc.object(),
      abitraryCallbackOptions(),
    ])(
      'Should set the correct values',
      (authToken, nodeUrl, component, value, configuration, callbacks) => {
        const notabene = new Notabene({
          nodeUrl,
          authToken,
        });
        const url = new URL(
          notabene.componentUrl(component, value, configuration, callbacks),
        );
        return (
          url.hash ===
            `#authToken=${encodeURIComponent(authToken)}&value=${encodeURIComponent(JSON.stringify(value))}&configuration=${encodeURIComponent(JSON.stringify(configuration))}` &&
          url.searchParams.get('nodeUrl') === nodeUrl &&
          url.pathname === `/${component}` &&
          url.searchParams.get('callback_url') === callbacks.callback &&
          url.searchParams.get('redirect_uri') === callbacks.redirectUri
        );
      },
    );
  });
});
