import EmbeddedComponent from './components/EmbeddedComponent';
import type {
  Agent,
  AgentType,
  CallbackOptions,
  ComponentMessage,
  ConnectionRequest,
  ConnectionResponse,
  Counterparty,
  DeclarationProof,
  Deposit,
  DepositRequest,
  Destination,
  HostMessage,
  IVMS101,
  MicroTransferProof,
  OwnershipProof,
  PersonType,
  ProofStatus,
  ProofTypes,
  ScreenshotProof,
  SignatureProof,
  Status,
  Theme,
  Transaction,
  TransactionAsset,
  TransactionOptions,
  VASP,
  Wallet,
  Withdrawal,
} from './types';
import { CMType, TransactionResponse } from './types';
import { encodeObjectToFragment } from './utils/urls';

export type {
  CMType,
  ComponentMessage,
  EmbeddedComponent,
  ConnectionRequest,
  ConnectionResponse,
  HostMessage,
  Theme,
  Counterparty,
  Withdrawal,
  Deposit,
  Transaction,
  DepositRequest,
  TransactionResponse,
  Destination as destination,
  TransactionAsset,
  SignatureProof,
  ScreenshotProof,
  MicroTransferProof,
  DeclarationProof,
  Notabene,
  ProofStatus,
  ProofTypes,
  IVMS101,
  PersonType,
  Status,
  VASP,
  Wallet,
  Agent,
  AgentType,
  OwnershipProof,
  TransactionOptions,
  CallbackOptions,
};

/**
 * Configuration for the Notabene SDK
 *
 * @public
 */
/**
 * Configuration for the Notabene SDK
 *
 * @public
 */
export interface NotabeneConfig {
  /**
   * The URL of the Notabene API node
   */
  nodeUrl?: string;

  /**
   * The authentication token for the Notabene API
   */
  authToken?: string;

  /**
   * The URL of the Notabene UX components
   */
  uxUrl?: string;

  /**
   * Custom theme configuration for the UX components
   */
  theme?: Theme;

  /**
   * The locale to use for the UX components
   */
  locale?: string;
}
/**
 * Primary constructor for Notabene UX elements
 *
 * This class provides methods to create and manage various Notabene components
 * such as withdrawal assist, deposit assist, connect, and deposit request.
 * It also handles URL generation and fragment decoding for these components.
 *
 * @public
 */
export default class Notabene {
  private nodeUrl?: string;
  private authToken?: string;
  private uxUrl: string;
  private theme?: Theme;
  private locale?: string;

  /**
   * Creates a new instance of the Notabene SDK
   *
   * @param config - Configuration options for the Notabene SDK
   */
  constructor(config: NotabeneConfig) {
    this.uxUrl = config.uxUrl || 'https://connect.notabene.id';
    this.nodeUrl = config.nodeUrl;
    this.authToken = config.authToken;
    this.theme = config.theme;
    this.locale = config.locale;
  }

  /**
   * Generates a URL for a Notabene component
   *
   * @param path - The path of the component
   * @param value - Transaction data
   * @param configuration - Optional transaction configuration
   * @param callbacks - Optional callback configuration
   * @returns component URL
   * @internal
   */
  componentUrl<V>(
    path: string,
    value: V,
    configuration?: TransactionOptions,
    callbacks?: CallbackOptions,
  ): string {
    const url = new URL(this.uxUrl);
    url.pathname = path;

    const hash = encodeObjectToFragment({
      authToken: this.authToken,
      value,
      configuration,
    });
    url.hash = hash;
    if (this.nodeUrl) url.searchParams.set('nodeUrl', this.nodeUrl);

    if (this.theme) {
      url.searchParams.set('theme', JSON.stringify(this.theme));
    }
    if (this.locale) {
      url.searchParams.set('locale', this.locale);
    }
    if (callbacks) {
      if (callbacks.callback)
        url.searchParams.set('callback_url', callbacks.callback);
      if (callbacks.redirectUri)
        url.searchParams.set('redirect_uri', callbacks.redirectUri);
    }
    return url.toString();
  }

  /**
   * Creates a new embedded component
   *
   * @param path - The path of the component
   * @param value - Transaction data
   * @param options - Optional transaction options
   * @param callbacks - Optional callback configuration
   * @returns A new EmbeddedComponent instance
   * @internal
   */
  createComponent<V, R>(
    path: string,
    value: V,
    options?: TransactionOptions,
    callbacks?: CallbackOptions,
  ) {
    return new EmbeddedComponent<V, R>(
      this.componentUrl(path, value, options, callbacks),
      value,
    );
  }

  /**
   * Creates a withdrawal assist component
   *
   * @param value - Withdrawal transaction data
   * @param options - Optional transaction options
   * @param callbacks - Optional callback configuration
   * @returns A new EmbeddedComponent instance for withdrawal assistance
   */
  public createWithdrawalAssist(
    value: Partial<Withdrawal>,
    options?: TransactionOptions,
    callbacks?: CallbackOptions,
  ) {
    return this.createComponent<
      Partial<Withdrawal>,
      TransactionResponse<Withdrawal>
    >('withdrawal-assist', value, options, callbacks);
  }

  /**
   * Creates a deposit assist component
   *
   * @param value - Deposit transaction data
   * @param options - Optional transaction options
   * @param callbacks - Optional callback configuration
   * @returns A new EmbeddedComponent instance for deposit assistance
   * @alpha
   */
  public createDepositAssist(
    value: Partial<Deposit>,
    options?: any,
    callbacks?: CallbackOptions,
  ) {
    return this.createComponent<Partial<Deposit>, TransactionResponse<Deposit>>(
      'deposit-assist',
      value,
      options,
      callbacks,
    );
  }

  /**
   * Creates a connect component
   *
   * @param value - Connection request data
   * @param options - Optional transaction options
   * @param callbacks - Optional callback configuration
   * @returns A new EmbeddedComponent instance for connection
   */
  public createConnectWallet(
    value: ConnectionRequest,
    options?: TransactionOptions,
    callbacks?: CallbackOptions,
  ) {
    return this.createComponent<ConnectionRequest, ConnectionResponse>(
      'connect',
      value,
      options,
      callbacks,
    );
  }

  /**
   * Creates a deposit request component
   *
   * @param value - Deposit request data
   * @param options - Optional transaction options
   * @param callbacks - Optional callback configuration
   * @returns A new EmbeddedComponent instance for deposit requests
   * @alpha
   */
  public createDepositRequest(
    value: DepositRequest,
    options?: TransactionOptions,
    callbacks?: CallbackOptions,
  ) {
    return this.createComponent<
      DepositRequest,
      TransactionResponse<DepositRequest>
    >('deposit-request', value, options, callbacks);
  }

  /**
   * Decodes a URL fragment into an object
   *
   * @param fragment - The URL fragment to decode
   * @returns An object containing the decoded key-value pairs
   */
  public decodeFragmentToObject(fragment: string): Record<string, string> {
    const pairs = fragment.slice(1).split('&');
    return pairs.reduce(
      (obj, pair) => {
        const [key, value] = pair.split('=');
        obj[decodeURIComponent(key)] = decodeURIComponent(value);
        return obj;
      },
      {} as Record<string, string>,
    );
  }
}
